@@ -39,7 +39,6 @@ class TMDAlgo(AbstractAlgo):
         params (dict): The parameters required for growth. It should include the
             ``branching_method`` selected from: ``|bio_oriented, symmetric, directional]``.
         start_point (list[float]): The first point of the trunk.
-        skip_validation (bool): If set to ``False``, the parameters and distributions and
             the "min_bar_length" parameter are validated.
         context (Any): An object containing contextual information.
         random_generator (numpy.random.Generator): The random number generator to use.
@@ -50,7 +49,6 @@ class TMDAlgo(AbstractAlgo):
         input_data,
         params,
         start_point,
-        skip_validation=False,
         context=None,
         random_generator=np.random,
         **_,
@@ -58,24 +56,11 @@ class TMDAlgo(AbstractAlgo):
         """TMD basic grower."""
         super().__init__(input_data, params, start_point, context)
         self.bif_method = bif_methods[params["branching_method"]]
-        self.params = copy.deepcopy(params)
         self.ph_angles = self.select_persistence(input_data, random_generator)
         self.barcode = Barcode(list(self.ph_angles))
-        self.start_point = start_point
         self.apical_section = None
         self.apical_point_distance_from_soma = 0.0
         self.persistence_length = self.barcode.get_persistence_length()
-        # Validate parameters and distributions
-        if not skip_validation:
-            # Consistency check between parameters - persistence diagram
-            barSZ = input_data["min_bar_length"]
-            stepSZ = params["step_size"]["norm"]["mean"]
-            if stepSZ >= barSZ:
-                L.warning(
-                    "Selected step size %f is too big for bars of size %f",
-                    stepSZ,
-                    barSZ,
-                )
 
     def select_persistence(self, input_data, random_generator=np.random):
         """Select the persistence.
