@@ -20,7 +20,8 @@ import json
 import logging
 
 import numpy as np
-from morphio.mut import Morphology  # pylint: disable=import-error
+from diameter_synthesis import build_diameters
+from morphio.mut import Morphology
 from numpy.random import BitGenerator
 from numpy.random import Generator
 from numpy.random import RandomState
@@ -29,14 +30,16 @@ from numpy.random import SeedSequence
 from neurots.generate import diametrizer
 from neurots.generate import orientations as _oris
 from neurots.generate.orientations import OrientationManager
+from neurots.generate.orientations import check_3d_angles
 from neurots.generate.soma import Soma
 from neurots.generate.soma import SomaGrower
 from neurots.generate.tree import TreeGrower
 from neurots.morphmath import sample
 from neurots.morphmath.utils import normalize_vectors
+from neurots.preprocess import preprocess_inputs
+from neurots.utils import NeuroTSError
 from neurots.utils import convert_from_legacy_neurite_type
-from neurots.validator import validate_neuron_distribs
-from neurots.validator import validate_neuron_params
+from neurots.utils import point_to_section_segment
 
 L = logging.getLogger(__name__)
 
@@ -65,8 +68,8 @@ class NeuronGrower:
         input_distributions (dict): Distributions extracted from biological data.
         context (Any): An object containing contextual information.
         external_diametrizer (Callable): Diametrizer function for external diametrizer module
-        skip_validation (bool): If set to ``False``, the parameters and distributions are
-            validated.
+        skip_proprocessing (bool): If set to ``False``, the parameters and distributions are
+            preprocessed with registered validator and preprocessors.
         rng_or_seed (int or numpy.random.Generator): A random number generator to use. If an
             int is given, it is passed to :func:`numpy.random.default_rng()` to create a new
             random number generator.
@@ -81,14 +84,13 @@ class NeuronGrower:
         input_distributions,
         context=None,
         external_diametrizer=None,
-        skip_validation=False,
+        skip_preprocessing=False,
         rng_or_seed=np.random,
         trunk_orientations_class=OrientationManager,
     ):
         """Constructor of the NeuronGrower class."""
         self.neuron = Morphology()
         self.context = context
-        self.skip_validation = skip_validation
         if rng_or_seed is None or isinstance(
             rng_or_seed, (int, np.integer, SeedSequence, BitGenerator)
         ):
@@ -105,35 +107,12 @@ class NeuronGrower:
         L.debug("Input Parameters: %s", self.input_parameters)
         self.input_distributions = _load_json(input_distributions)
 
-        # Validate parameters and distributions
-        if not skip_validation:
-            self.validate_params()
-            self.validate_distribs()
-
-        # Consistency check between parameters and distributions
-        for tree_type in self.input_parameters["grow_types"]:
-            metric1 = self.input_parameters[tree_type].get("metric")
-            metric2 = self.input_distributions[tree_type].get("filtration_metric")
-            if metric1 not in ["trunk_length", metric2]:
-                raise ValueError(
-                    "Metric of parameters and distributions is inconsistent:"
-                    + f" {metric1} != {metric2}"
-                )
-
-        method1 = self.input_parameters["diameter_params"]["method"]
-        method2 = self.input_distributions["diameter"]["method"]
-        if method1 != method2:
-            raise ValueError(
-                "Diameters methods of parameters and distributions is inconsistent:"
-                + f" {method1} != {method2}"
+        # Validate and preprocess parameters and distributions
+        if not skip_preprocessing:
+            self.input_parameters, self.input_distributions = preprocess_inputs(
+                self.input_parameters, self.input_distributions
             )
 
-        if (
-            self.input_distributions["diameter"]["method"] == "external"
-            and external_diametrizer is None
-        ):
-            raise ValueError("External diametrizer is missing the diametrizer function.")
-
         # A list of trees with the corresponding orientations
         # and initial points on the soma surface will be initialized.
         self.active_neurites = []
@@ -155,14 +134,6 @@ class NeuronGrower:
 
         self._trunk_orientations_class = trunk_orientations_class
 
-    def validate_params(self):
-        """Validate the parameter dictionary."""
-        validate_neuron_params(self.input_parameters)
-
-    def validate_distribs(self):
-        """Validate the distribution dictionary."""
-        validate_neuron_distribs(self.input_distributions)
-
     def next(self):
         """Call the "next" method of each neurite grower."""
         for grower in list(self.active_neurites):
@@ -198,17 +169,38 @@ class NeuronGrower:
 
     def _post_grow(self):
         """Actions after the morphology has been grown and before its diametrization."""
+        # ensures section.id are consistent with morphio loader
+        apical_points = [
+            self.neuron.sections[apical_section].points[-1] if apical_section is not None else None
+            for apical_section in self.apical_sections
+        ]
+
+        self.neuron = Morphology(self.neuron)
+
+        self.apical_sections = [
+            point_to_section_segment(self.neuron, apical_point)[0]
+            if apical_point is not None
+            else None
+            for apical_point in apical_points
+        ]
 
     def _init_diametrizer(self, external_diametrizer=None):
         """Set a diametrizer function."""
-        if self.input_distributions["diameter"]["method"] == "default":
+        if (
+            self.input_distributions["diameter"]["method"] == "external"
+            and external_diametrizer is None
+        ):
+            raise ValueError("External diametrizer is missing the diametrizer function.")
+
+        if self.input_distributions["diameter"]["method"] == "no_diameters":
             self._diametrize = lambda: None
             L.warning("No diametrizer provided, so neurons will have default diameters.")
         else:
             if self.input_distributions["diameter"]["method"] == "external":
-                if external_diametrizer is None:
-                    raise Exception("Please provide an external diametrizer!")
                 diam_method = external_diametrizer
+            elif self.input_distributions["diameter"]["method"] == "default":
+                self.input_parameters["diameter_params"]["models"] = ["simpler"]
+                diam_method = build_diameters.build
             else:
                 diam_method = self.input_distributions["diameter"]["method"]
 
@@ -249,7 +241,6 @@ class NeuronGrower:
             ), "Orientations should have non-zero lengths"
             if params.get("trunk_absolute_orientation", False):
                 if len(orientation) == 1:
-
                     orientation = np.asarray(orientation[0], dtype=np.float64)
                     orientation /= np.linalg.norm(orientation)
 
@@ -267,7 +258,6 @@ class NeuronGrower:
                     raise ValueError("The orientation should contain exactly one point!")
             else:
                 if len(orientation) >= n_trees:
-
                     # TODO: pick orientations randomly?
                     orientations = normalize_vectors(np.asarray(orientation, dtype=np.float64))[
                         :n_trees
@@ -301,10 +291,10 @@ class NeuronGrower:
         pts = self.soma_grower.add_points_from_orientations(orientations)
         return pts
 
-    def _grow_trunks(self):
+    def _simple_grow_trunks(self):
         """Grow the trunks.
 
-        Generates the initial points of each tree, which depend on the selectedS
+        Generates the initial points of each tree, which depend on the selected
         tree types and the soma surface. All the trees start growing from the surface
         of the soma. The outgrowth direction is either specified in the input parameters,
         as ``parameters['type']['orientation']`` or it is randomly chosen according to the
@@ -315,7 +305,6 @@ class NeuronGrower:
         legacy_mode = not isinstance(self.input_parameters[tree_types[0]]["orientation"], dict)
 
         if not legacy_mode:
-
             trunk_orientations_manager = self._trunk_orientations_class(
                 soma=self.soma_grower.soma,
                 parameters=self.input_parameters,
@@ -325,29 +314,30 @@ class NeuronGrower:
             )
 
         for type_of_tree in tree_types:
-
             params = self.input_parameters[type_of_tree]
             distr = self.input_distributions[type_of_tree]
 
             if legacy_mode:
-
                 n_trees = sample.n_neurites(distr["num_trees"], random_generator=self._rng)
 
                 if type_of_tree == "basal_dendrite" and n_trees < 2:
-                    raise Exception(f"There should be at least 2 basal dendrites (got {n_trees})")
+                    raise NeuroTSError(
+                        f"There should be at least 2 basal dendrites (got {n_trees})"
+                    )
 
                 orientation = params["orientation"]
                 points = self._convert_orientation2points(orientation, n_trees, distr, params)
 
             else:
-
                 orientations = trunk_orientations_manager.compute_tree_type_orientations(
                     type_of_tree
                 )
                 n_trees = len(orientations)
 
                 if type_of_tree == "basal_dendrite" and n_trees < 2:
-                    raise Exception(f"There should be at least 2 basal dendrites (got {n_trees})")
+                    raise NeuroTSError(
+                        f"There should be at least 2 basal dendrites (got {n_trees})"
+                    )
 
                 points = self.soma_grower.add_points_from_orientations(orientations)
 
@@ -361,12 +351,52 @@ class NeuronGrower:
                         initial_point=p,
                         parameters=params,
                         distributions=distr,
-                        skip_validation=self.skip_validation,
                         context=self.context,
                         random_generator=self._rng,
                     )
                 )
 
+    def _3d_angles_grow_trunks(self):
+        """Grow trunk with 3d_angles method via :func:`.orientation.OrientationManager` class.
+
+        Args:
+            input_parameters_with_3d_anglles (dict): input_parameters with fits for 3d angles
+        """
+        trunk_orientations_manager = self._trunk_orientations_class(
+            soma=self.soma_grower.soma,
+            parameters=self.input_parameters,
+            distributions=self.input_distributions,
+            context=self.context,
+            rng=self._rng,
+        )
+        for neurite_type in self.input_parameters["grow_types"]:
+            orientations = trunk_orientations_manager.compute_tree_type_orientations(neurite_type)
+
+            for p in self.soma_grower.add_points_from_orientations(orientations):
+                self.active_neurites.append(
+                    TreeGrower(
+                        self.neuron,
+                        initial_direction=self.soma_grower.soma.orientation_from_point(p),
+                        initial_point=p,
+                        parameters=self.input_parameters[neurite_type],
+                        distributions=self.input_distributions[neurite_type],
+                        context=self.context,
+                        random_generator=self._rng,
+                    )
+                )
+
+    def _grow_trunks(self):
+        """Grow the trunks.
+
+        Two methods are available, depending on the data present in the `input_parameters`.
+        If no `3d_angles` entry is present, we grow trunks with :func:`_simple_grow_trunks` else
+        we fit the raw binned 3d angle data and apply :func:`_3d_angles_grow_trunks`.
+        """
+        if check_3d_angles(self.input_parameters):
+            self._3d_angles_grow_trunks()
+        else:
+            self._simple_grow_trunks()
+
     def _grow_soma(self, soma_type="contour"):
         """Generates a soma based on the input_distributions.
 
