@@ -19,6 +19,9 @@ import warnings
 from copy import deepcopy
 
 import numpy as np
+from neurom import COLS
+
+PIA_DIRECTION = [0.0, 1.0, 0.0]
 
 
 class NeuroTSError(Exception):
@@ -56,7 +59,6 @@ def convert_from_legacy_neurite_type(data):
     """Convert legacy neurite type names, basal -> basal_dendrite and apical -> apical_dendrite."""
     old_data = deepcopy(data)
     for key, _data in old_data.items():
-
         if key == "apical":
             neurite_type_warning(key)
             data["apical_dendrite"] = data.pop("apical")
@@ -79,3 +81,28 @@ def convert_from_legacy_neurite_type(data):
                     data[key][i] = "basal_dendrite"
 
     return data
+
+
+def point_to_section_segment(neuron, point, rtol=1e-05, atol=1e-08):
+    """Find section and segment that matches the point (also in morph_tool.spatial).
+
+    Only the first point found with the *exact* same coordinates as the point argument is considered
+
+    Args:
+        neuron (morphio.Morphology): neuron object
+        point (point): value of the point to find in the h5 file
+        rtol, atol (floats): precision of np.isclose
+
+    Returns:
+        Tuple: (NeuroM/MorphIO section ID, point ID) of the point the matches the input coordinates.
+        Since NeuroM v2, section ids of NeuroM and MorphIO are the same excluding soma.
+    """
+    for section in neuron.iter():
+        points = section.points
+        offset = np.where(
+            np.isclose(points[:, COLS.XYZ], point[COLS.XYZ], rtol=rtol, atol=atol).all(axis=1)
+        )
+        if offset[0].size:
+            return section.id, offset[0][0]
+
+    raise ValueError(f"Cannot find point in morphology that matches: {point}")
