@@ -1,4 +1,4 @@
-"""Tests for the NeuroTS package."""
+"""Tests suite for the NeuroTS package."""
 
 # Copyright (C) 2021  Blue Brain Project, EPFL
 #
