@@ -29,14 +29,12 @@ from .test_grower import _parameters
 
 
 def test_tree_grower_space_colonization_constructor():
-
     neuron = Mock()
     initial_direction = np.array([0.12427115, 0.93206836, 0.3403017])
     initial_point = np.array([1.5411615, 0.29389329, 0.15904417])
 
     grower_distributions = _distributions()
     grower_parameters = _parameters()
-    skip_validation = False
     grower_context = SpaceColonizationContext(_context())
     tree_parameters = grower_parameters["basal"]
     tree_parameters["origin"] = np.zeros(3)
@@ -48,7 +46,6 @@ def test_tree_grower_space_colonization_constructor():
         initial_point,
         tree_parameters,
         tree_distributions,
-        skip_validation,
         grower_context,
     )
 
@@ -66,7 +63,6 @@ def test_tree_grower_space_colonization_constructor():
         initial_point,
         tree_parameters,
         tree_distributions,
-        skip_validation,
         grower_context,
     )
     assert isinstance(tree_grower.active_sections[0], SectionSpatialGrower)
