@@ -17,6 +17,7 @@
 
 # pylint: disable=missing-function-docstring
 # pylint: disable=redefined-outer-name
+# pylint: disable=protected-access
 import os
 
 import neurom
@@ -24,14 +25,16 @@ import numpy as np
 import pytest
 import tmd
 from neurom import load_morphologies
-from neurom import stats
 from numpy.testing import assert_array_almost_equal
 from numpy.testing import assert_equal
+from pkg_resources import parse_version
 
 from neurots import NeuroTSError
 from neurots import extract_input
 from neurots import validator
 
+_OLD_NUMPY = parse_version(np.__version__) < parse_version("1.21")
+
 _PATH = os.path.join(os.path.dirname(os.path.abspath(__file__)), "..", "test_data")
 POP_PATH = os.path.join(_PATH, "bio/")
 NEU_PATH = os.path.join(_PATH, "diam_simple.swc")
@@ -59,7 +62,7 @@ def test_num_trees(POPUL):
     assert_equal(numAX, target_numAX)
 
 
-def test_trunk_distr(POPUL):
+def test_trunk_distr(POPUL, NEU):
     bins_BAS = [
         0.19391773616376634,
         0.4880704446023673,
@@ -97,6 +100,38 @@ def test_trunk_distr(POPUL):
             "azimuth": {"uniform": {"max": 0.0, "min": np.pi}},
             "orientation_deviation": {"data": {"weights": [4, 3, 1, 2, 0, 1, 0, 0, 0, 2]}},
             "absolute_elevation_deviation": absolute_elevation_deviation_BAS,
+            "pia_3d_angles": {
+                "data": {
+                    "weights": [
+                        1.2274214110745432,
+                        0.6137107055372726,
+                        0.6137107055372716,
+                        0.6137107055372716,
+                        1.2274214110745432,
+                        0.6137107055372716,
+                        1.841132116611821,
+                        0.0,
+                        0.0,
+                        1.2274214110745432,
+                    ]
+                }
+            },
+            "apical_3d_angles": {
+                "data": {
+                    "weights": [
+                        1.0248077967009541,
+                        0.0,
+                        1.0248077967009541,
+                        2.0496155934019082,
+                        0.5124038983504771,
+                        1.0248077967009541,
+                        0.0,
+                        0.0,
+                        0.5124038983504771,
+                        0.5124038983504771,
+                    ]
+                }
+            },
         }
     }
     target_trunkAPIC = {
@@ -104,6 +139,7 @@ def test_trunk_distr(POPUL):
             "azimuth": {"uniform": {"max": 0.0, "min": np.pi}},
             "orientation_deviation": {"data": {"bins": [0.0], "weights": [2]}},
             "absolute_elevation_deviation": {"data": {"weights": [2]}},
+            "pia_3d_angles": {"data": {"weights": [8.9044]}},
         }
     }
 
@@ -111,6 +147,8 @@ def test_trunk_distr(POPUL):
         POPUL, neurite_type=neurom.APICAL_DENDRITE, bins=1
     )
     trunkBAS = extract_input.from_neurom.trunk_neurite(POPUL, bins=10)
+    trunkNEU = extract_input.from_neurom.trunk_neurite(NEU, bins=10)
+    assert "apical_3d_angles" not in trunkNEU
 
     assert_array_almost_equal(trunkBAS["trunk"]["orientation_deviation"]["data"]["bins"], bins_BAS)
     assert_array_almost_equal(
@@ -123,9 +161,16 @@ def test_trunk_distr(POPUL):
     )
     del trunkBAS["trunk"]["orientation_deviation"]["data"]["bins"]
     del trunkBAS["trunk"]["absolute_elevation_deviation"]["data"]["bins"]
+    del trunkBAS["trunk"]["pia_3d_angles"]["data"]["bins"]
+    del trunkBAS["trunk"]["apical_3d_angles"]["data"]["bins"]
     del trunkAP["trunk"]["absolute_elevation_deviation"]["data"]["bins"]
+    del trunkAP["trunk"]["pia_3d_angles"]["data"]["bins"]
 
     assert_equal(trunkBAS, target_trunkBAS)
+    # this value is slightly unstable with python versions
+    trunkAP["trunk"]["pia_3d_angles"]["data"]["weights"][0] = np.around(
+        trunkAP["trunk"]["pia_3d_angles"]["data"]["weights"][0], 4
+    )
     assert_equal(trunkAP, target_trunkAPIC)
 
 
@@ -295,6 +340,14 @@ class TestDistributions:
         )
         validator.validate_neuron_distribs(distr)
 
+    def test_diameter_model_invalid(self, filename):
+        with pytest.raises(NotImplementedError):
+            extract_input.distributions(
+                filename,
+                feature="radial_distances",
+                diameter_model="invalid",
+            )
+
     def test_diameter_model_M5(self, filename):
         distr_M5 = extract_input.distributions(
             filename, feature="radial_distances", diameter_model="M5"
@@ -326,24 +379,6 @@ class TestDistributions:
         assert distr_external == distr_external_input
 
 
-def test_transform_distr():
-    ss = stats.fit([1, 2], distribution="norm")
-    tss = extract_input.from_neurom.transform_distr(ss)
-    assert_equal(tss, {"norm": {"mean": 1.5, "std": 0.5}})
-
-    ss = stats.fit([1, 2], distribution="uniform")
-    tss = extract_input.from_neurom.transform_distr(ss)
-    assert_equal(tss, {"uniform": {"min": 1, "max": 2}})
-
-    ss = stats.fit([1, 2], distribution="expon")
-    tss = extract_input.from_neurom.transform_distr(ss)
-    assert_equal(tss, {"expon": {"loc": 1.0, "lambda": 2.0}})
-
-    ss = stats.fit([1, 2], distribution="gamma")
-    tss = extract_input.from_neurom.transform_distr(ss)
-    assert_equal(tss, None)
-
-
 def test_number_neurites(POPUL):
     res = extract_input.from_neurom.number_neurites(POPUL)
     assert_equal(
@@ -407,7 +442,7 @@ def test_parameters():
         "axon": {},
         "origin": [0.0, 0.0, 0.0],
         "grow_types": ["basal_dendrite", "apical_dendrite"],
-        "diameter_params": {"method": "default"},
+        "diameter_params": {"method": "default", "models": ["simpler"]},
     }
     assert_equal(params, expected_params)
 
@@ -470,7 +505,7 @@ def test_parameters():
             "axon": {},
             "origin": [0.0, 0.0, 0.0],
             "grow_types": ["basal_dendrite", "apical_dendrite"],
-            "diameter_params": {"method": "default"},
+            "diameter_params": {"method": "default", "models": ["simpler"]},
         },
     )
     validator.validate_neuron_params(params_path)
@@ -496,7 +531,7 @@ def test_parameters():
             },
             "origin": [0.0, 0.0, 0.0],
             "grow_types": ["axon"],
-            "diameter_params": {"method": "default"},
+            "diameter_params": {"method": "default", "models": ["simpler"]},
         },
     )
     validator.validate_neuron_params(params_axon)
@@ -522,7 +557,7 @@ def test_parameters():
             },
             "origin": [0.0, 0.0, 0.0],
             "grow_types": ["axon"],
-            "diameter_params": {"method": "default"},
+            "diameter_params": {"method": "default", "models": ["simpler"]},
         },
     )
     validator.validate_neuron_params(params)
@@ -597,6 +632,54 @@ def test_parameters():
             diameter_parameters=object(),
         )
 
+    params = extract_input.parameters(neurite_types=["basal_dendrite", "apical_dendrite"])
+    assert_equal(
+        params,
+        {
+            "basal_dendrite": {
+                "randomness": 0.24,
+                "targeting": 0.14,
+                "radius": 0.3,
+                "orientation": None,
+                "growth_method": "tmd",
+                "branching_method": "bio_oriented",
+                "modify": None,
+                "step_size": {"norm": {"mean": 1.0, "std": 0.2}},
+                "metric": "path_distances",
+                "tree_type": 3,
+            },
+            "apical_dendrite": {
+                "randomness": 0.24,
+                "targeting": 0.14,
+                "radius": 0.3,
+                "orientation": [[0.0, 1.0, 0.0]],
+                "growth_method": "tmd_apical",
+                "branching_method": "directional",
+                "modify": None,
+                "step_size": {"norm": {"mean": 1.0, "std": 0.2}},
+                "metric": "path_distances",
+                "tree_type": 4,
+            },
+            "axon": {},
+            "origin": [0.0, 0.0, 0.0],
+            "grow_types": ["basal_dendrite", "apical_dendrite"],
+            "diameter_params": {"method": "default", "models": ["simpler"]},
+        },
+    )
+    validator.validate_neuron_params(params_path)
+
+    extract_input.parameters(
+        neurite_types=["axon"],
+        method="trunk",
+        diameter_parameters={"some_external_diametrizer": None},
+    )
+
+    extract_input.parameters(
+        neurite_types=["axon"],
+        method="trunk",
+        diameter_parameters="some_diametrizer",
+    )
+
 
 def test_from_TMD():
     files = sorted([os.path.join(POP_PATH, neuron_dir) for neuron_dir in os.listdir(POP_PATH)])
@@ -666,7 +749,7 @@ def test_from_TMD():
     ]
     for a, b in zip(angles["persistence_diagram"], expected):
         for ai, bi in zip(a, b):
-            assert_array_almost_equal(ai, bi)
+            assert_array_almost_equal(ai, bi, decimal=6 if not _OLD_NUMPY else 4)
 
     angles = extract_input.from_TMD.persistent_homology_angles(
         pop, neurite_type="basal_dendrite", threshold=9
@@ -699,4 +782,79 @@ def test_from_TMD():
     ]
     for a, b in zip(angles["persistence_diagram"], expected):
         for ai, bi in zip(a, b):
-            assert_array_almost_equal(ai, bi)
+            assert_array_almost_equal(ai, bi, decimal=6 if not _OLD_NUMPY else 5)
+
+
+def test_trunk_neurite_3d_angles(POPUL):
+    all_angles = extract_input.from_neurom.trunk_neurite(POPUL, neurom.APICAL_DENDRITE, bins=10)
+    angles = extract_input.from_neurom.trunk_neurite_3d_angles(
+        POPUL, neurom.APICAL_DENDRITE, bins=10
+    )
+
+    assert all_angles["trunk"]["pia_3d_angles"] == angles["trunk"]["pia_3d_angles"]
+    assert_array_almost_equal(
+        angles["trunk"]["pia_3d_angles"]["data"]["bins"],
+        [
+            0.4828722567123299,
+            0.49410265776120743,
+            0.5053330588100851,
+            0.5165634598589626,
+            0.5277938609078402,
+            0.5390242619567178,
+            0.5502546630055953,
+            0.5614850640544728,
+            0.5727154651033505,
+            0.5839458661522281,
+        ],
+    )
+    assert_array_almost_equal(
+        angles["trunk"]["pia_3d_angles"]["data"]["weights"],
+        [44.52200752438604, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 44.52200752438582],
+    )
+
+    angles = extract_input.from_neurom.trunk_neurite(POPUL, neurom.BASAL_DENDRITE, bins=10)
+    assert angles["trunk"]["pia_3d_angles"] == {
+        "data": {
+            "bins": [
+                1.2145526099759574,
+                1.3398935503066864,
+                1.4652344906374157,
+                1.590575430968145,
+                1.7159163712988743,
+                1.8412573116296036,
+                1.9665982519603327,
+                2.0919391922910617,
+                2.217280132621791,
+                2.3426210729525203,
+            ],
+            "weights": [
+                1.2274214110745432,
+                0.6137107055372726,
+                0.6137107055372716,
+                0.6137107055372716,
+                1.2274214110745432,
+                0.6137107055372716,
+                1.841132116611821,
+                0.0,
+                0.0,
+                1.2274214110745432,
+            ],
+        }
+    }
+
+
+def test_transform_distr():
+    np.random.seed(42)
+    data = np.random.uniform(0, 1, 100)
+
+    ss = neurom.stats.fit(data, distribution="norm")
+    res = extract_input.from_neurom.transform_distr(ss)
+    assert_equal(res, {"norm": {"mean": 0.47018074337820936, "std": 0.29599822663249037}})
+
+    ss = neurom.stats.fit(data, distribution="uniform")
+    res = extract_input.from_neurom.transform_distr(ss)
+    assert_equal(res, {"uniform": {"min": 0.005522117123602399, "max": 0.9868869366005173}})
+
+    ss = neurom.stats.fit(data, distribution="expon")
+    res = extract_input.from_neurom.transform_distr(ss)
+    assert_equal(res, {"expon": {"loc": 0.005522117123602399, "lambda": 2.1521175837421254}})
