@@ -42,6 +42,7 @@ from scipy.spatial.distance import cdist
 
 from neurots.generate.diametrizer import diametrize_constant_per_neurite
 from neurots.generate.grower import NeuronGrower
+from neurots.preprocess.exceptions import NeuroTSValidationError
 
 _path = os.path.join(os.path.dirname(os.path.abspath(__file__)), "data")
 
@@ -81,7 +82,7 @@ def _test_full(
     ref_persistence_diagram,
     save=False,
     rng_or_seed=None,
-    skip_validation=False,
+    skip_preprocessing=False,
 ):
     distributions, params = _load_inputs(join(_path, distributions), join(_path, parameters))
     if rng_or_seed is None:
@@ -89,14 +90,14 @@ def _test_full(
         n = NeuronGrower(
             input_distributions=distributions,
             input_parameters=params,
-            skip_validation=skip_validation,
+            skip_preprocessing=skip_preprocessing,
         ).grow()
     else:
         n = NeuronGrower(
             input_distributions=distributions,
             input_parameters=params,
             rng_or_seed=rng_or_seed,
-            skip_validation=skip_validation,
+            skip_preprocessing=skip_preprocessing,
         ).grow()
 
     with TemporaryDirectory("test_grower") as folder:
@@ -206,10 +207,6 @@ def test_external_diametrizer():
     ):
         NeuronGrower(parameters, distributions)
 
-    bad_ng = NeuronGrower(parameters, distributions, external_diametrizer=object())
-    with pytest.raises(Exception, match="Please provide an external diametrizer!"):
-        bad_ng._init_diametrizer()
-
     # Test with an external diametrizer and neurite_types in diameter_params
     distributions["diameter"]["method"] = "external"
     parameters["diameter_params"]["method"] = "external"
@@ -291,14 +288,14 @@ def test_breaker_of_tmd_algo():
     N = NeuronGrower(input_distributions=distributions, input_parameters=params)
     n = N.grow()
 
-    assert_array_equal(N.apical_sections, [33])
+    assert_array_equal(N.apical_sections, [11])
     assert_array_almost_equal(
-        n.sections[169].points[-1],
+        n.sections[118].points[-1],
         np.array([-220.93813, -21.49141, -55.93323]),
         decimal=5,
     )
     assert_array_almost_equal(
-        n.sections[122].points[-1], np.array([-17.31787, 151.4876, -6.67741]), decimal=5
+        n.sections[30].points[-1], np.array([-17.31787, 151.4876, -6.67741]), decimal=5
     )
 
     # Test with a specific random generator
@@ -307,14 +304,14 @@ def test_breaker_of_tmd_algo():
     N = NeuronGrower(input_distributions=distributions, input_parameters=params, rng_or_seed=rng)
     n = N.grow()
 
-    assert_array_equal(N.apical_sections, [33])
+    assert_array_equal(N.apical_sections, [11])
     assert_array_almost_equal(
-        n.sections[169].points[-1],
+        n.sections[118].points[-1],
         np.array([-220.93813, -21.49141, -55.93323]),
         decimal=5,
     )
     assert_array_almost_equal(
-        n.sections[122].points[-1], np.array([-17.31787, 151.4876, -6.67741]), decimal=5
+        n.sections[30].points[-1], np.array([-17.31787, 151.4876, -6.67741]), decimal=5
     )
 
 
@@ -345,7 +342,7 @@ def test_axon_grower():
         "axon_trunk_parameters_orientation_manager.json",
         "test_axon_grower.h5",
         None,
-        skip_validation=True,
+        skip_preprocessing=True,
     )
     _test_full(
         "radial_distances",
@@ -354,7 +351,7 @@ def test_axon_grower():
         "test_axon_grower.h5",
         None,
         rng_or_seed=build_random_generator(0),
-        skip_validation=True,
+        skip_preprocessing=True,
     )
 
     _test_full(
@@ -378,7 +375,7 @@ def test_axon_grower():
         "axon_trunk_parameters_absolute_orientation_manager.json",
         "test_axon_grower_absolute.h5",
         None,
-        skip_validation=True,
+        skip_preprocessing=True,
     )
     _test_full(
         "radial_distances",
@@ -387,7 +384,7 @@ def test_axon_grower():
         "test_axon_grower_absolute.h5",
         None,
         rng_or_seed=build_random_generator(0),
-        skip_validation=True,
+        skip_preprocessing=True,
     )
 
 
@@ -462,27 +459,48 @@ class TestPathGrower:
             rng_or_seed=build_random_generator(0),
         )
 
-    def test_skip_validation(self):
+    def test_skip_preprocessing(self):
         _test_full(
             "path_distances",
             "bio_distribution.json",
             "bio_path_params_orientation_manager.json",
             "path_grower.h5",
             "bio_path_persistence_diagram.json",
-            skip_validation=True,
+            skip_preprocessing=True,
         )
 
-    def test_skip_rng_and_validation(self):
+    def test_skip_rng_and_preprocessing(self):
         _test_full(
             "path_distances",
             "bio_distribution.json",
             "bio_path_params_orientation_manager.json",
             "path_grower.h5",
             "bio_path_persistence_diagram.json",
-            skip_validation=True,
+            skip_preprocessing=True,
             rng_or_seed=build_random_generator(0),
         )
 
+    def test_min_bar_length_missing(self, tmpdir):
+        """Check that the proper exception is raised when the min_bar_length entry is missing."""
+        with open(join(_path, "bio_distribution.json"), encoding="utf-8") as f:
+            distributions = json.load(f)
+        del distributions["apical_dendrite"]["min_bar_length"]
+        broken_distribution = tmpdir / "bio_distribution.json"
+        with open(broken_distribution, mode="w", encoding="utf-8") as f:
+            json.dump(distributions, f)
+
+        with pytest.raises(
+            NeuroTSValidationError,
+            match="'min_bar_length'",
+        ):
+            _test_full(
+                "path_distances",
+                broken_distribution,
+                "bio_path_params.json",
+                "path_grower.h5",
+                "bio_path_persistence_diagram.json",
+            )
+
 
 class TestGradientPathGrower:
     """test tmd_path"""
@@ -506,24 +524,24 @@ class TestGradientPathGrower:
             rng_or_seed=build_random_generator(0),
         )
 
-    def test_skip_validation(self):
+    def test_skip_preprocessing(self):
         _test_full(
             "path_distances",
             "bio_distribution.json",
             "bio_gradient_path_params_orientation_manager.json",
             "gradient_path_grower.h5",
             "gradient_path_persistence_diagram.json",
-            skip_validation=True,
+            skip_preprocessing=True,
         )
 
-    def test_skip_rng_and_validation(self):
+    def test_skip_rng_and_preprocessing(self):
         _test_full(
             "path_distances",
             "bio_distribution.json",
             "bio_gradient_path_params_orientation_manager.json",
             "gradient_path_grower.h5",
             "gradient_path_persistence_diagram.json",
-            skip_validation=True,
+            skip_preprocessing=True,
             rng_or_seed=build_random_generator(0),
         )
 
@@ -532,7 +550,7 @@ class TestBioRatL5Tpc1:
     def test_default(self):
         _test_full(
             "path_distances",
-            "bio_rat_L5_TPC_B.json",
+            "bio_rat_L5_TPC_B_distribution.json",
             "params1.json",
             "expected_bio_rat_L5_TPC_B_with_params1.h5",
             "expected_bio_rat_L5_TPC_B_with_params1_persistence_diagram.json",
@@ -541,31 +559,31 @@ class TestBioRatL5Tpc1:
     def test_with_rng(self):
         _test_full(
             "path_distances",
-            "bio_rat_L5_TPC_B.json",
+            "bio_rat_L5_TPC_B_distribution.json",
             "params1.json",
             "expected_bio_rat_L5_TPC_B_with_params1.h5",
             "expected_bio_rat_L5_TPC_B_with_params1_persistence_diagram.json",
             rng_or_seed=build_random_generator(0),
         )
 
-    def test_skip_validation(self):
+    def test_skip_preprocessing(self):
         _test_full(
             "path_distances",
-            "bio_rat_L5_TPC_B.json",
+            "bio_rat_L5_TPC_B_distribution.json",
             "params1_orientation_manager.json",
             "expected_bio_rat_L5_TPC_B_with_params1.h5",
             "expected_bio_rat_L5_TPC_B_with_params1_persistence_diagram.json",
-            skip_validation=True,
+            skip_preprocessing=True,
         )
 
-    def test_skip_rng_and_validation(self):
+    def test_skip_rng_and_preprocessing(self):
         _test_full(
             "path_distances",
-            "bio_rat_L5_TPC_B.json",
+            "bio_rat_L5_TPC_B_distribution.json",
             "params1_orientation_manager.json",
             "expected_bio_rat_L5_TPC_B_with_params1.h5",
             "expected_bio_rat_L5_TPC_B_with_params1_persistence_diagram.json",
-            skip_validation=True,
+            skip_preprocessing=True,
             rng_or_seed=build_random_generator(0),
         )
 
@@ -574,7 +592,7 @@ class TestBioRatL5Tpc2:
     def test_default(self):
         _test_full(
             "path_distances",
-            "bio_rat_L5_TPC_B.json",
+            "bio_rat_L5_TPC_B_distribution.json",
             "params2.json",
             "expected_bio_rat_L5_TPC_B_with_params2.h5",
             "expected_bio_rat_L5_TPC_B_with_params2_persistence_diagram.json",
@@ -583,31 +601,31 @@ class TestBioRatL5Tpc2:
     def test_with_rng(self):
         _test_full(
             "path_distances",
-            "bio_rat_L5_TPC_B.json",
+            "bio_rat_L5_TPC_B_distribution.json",
             "params2.json",
             "expected_bio_rat_L5_TPC_B_with_params2.h5",
             "expected_bio_rat_L5_TPC_B_with_params2_persistence_diagram.json",
             rng_or_seed=build_random_generator(0),
         )
 
-    def test_skip_validation(self):
+    def test_skip_preprocessing(self):
         _test_full(
             "path_distances",
-            "bio_rat_L5_TPC_B.json",
+            "bio_rat_L5_TPC_B_distribution.json",
             "params2_orientation_manager.json",
             "expected_bio_rat_L5_TPC_B_with_params2.h5",
             "expected_bio_rat_L5_TPC_B_with_params2_persistence_diagram.json",
-            skip_validation=True,
+            skip_preprocessing=True,
         )
 
-    def test_skip_rng_and_validation(self):
+    def test_skip_rng_and_preprocessing(self):
         _test_full(
             "path_distances",
-            "bio_rat_L5_TPC_B.json",
+            "bio_rat_L5_TPC_B_distribution.json",
             "params2_orientation_manager.json",
             "expected_bio_rat_L5_TPC_B_with_params2.h5",
             "expected_bio_rat_L5_TPC_B_with_params2_persistence_diagram.json",
-            skip_validation=True,
+            skip_preprocessing=True,
             rng_or_seed=build_random_generator(0),
         )
 
@@ -616,7 +634,7 @@ class TestBioRatL5Tpc3:
     def test_default(self):
         _test_full(
             "path_distances",
-            "bio_rat_L5_TPC_B.json",
+            "bio_rat_L5_TPC_B_distribution.json",
             "params3.json",
             "expected_bio_rat_L5_TPC_B_with_params3.h5",
             "expected_bio_rat_L5_TPC_B_with_params3_persistence_diagram.json",
@@ -625,31 +643,31 @@ class TestBioRatL5Tpc3:
     def test_with_rng(self):
         _test_full(
             "path_distances",
-            "bio_rat_L5_TPC_B.json",
+            "bio_rat_L5_TPC_B_distribution.json",
             "params3.json",
             "expected_bio_rat_L5_TPC_B_with_params3.h5",
             "expected_bio_rat_L5_TPC_B_with_params3_persistence_diagram.json",
             rng_or_seed=build_random_generator(0),
         )
 
-    def test_skip_validation(self):
+    def test_skip_preprocessing(self):
         _test_full(
             "path_distances",
-            "bio_rat_L5_TPC_B.json",
+            "bio_rat_L5_TPC_B_distribution.json",
             "params3_orientation_manager.json",
             "expected_bio_rat_L5_TPC_B_with_params3.h5",
             "expected_bio_rat_L5_TPC_B_with_params3_persistence_diagram.json",
-            skip_validation=True,
+            skip_preprocessing=True,
         )
 
-    def test_skip_rng_and_validation(self):
+    def test_skip_rng_and_preprocessing(self):
         _test_full(
             "path_distances",
-            "bio_rat_L5_TPC_B.json",
+            "bio_rat_L5_TPC_B_distribution.json",
             "params3_orientation_manager.json",
             "expected_bio_rat_L5_TPC_B_with_params3.h5",
             "expected_bio_rat_L5_TPC_B_with_params3_persistence_diagram.json",
-            skip_validation=True,
+            skip_preprocessing=True,
             rng_or_seed=build_random_generator(0),
         )
 
@@ -658,7 +676,7 @@ class TestBioRatL5Tpc4:
     def test_default(self):
         _test_full(
             "path_distances",
-            "bio_rat_L5_TPC_B.json",
+            "bio_rat_L5_TPC_B_distribution.json",
             "params4.json",
             "expected_bio_rat_L5_TPC_B_with_params4.h5",
             "expected_bio_rat_L5_TPC_B_with_params4_persistence_diagram.json",
@@ -667,30 +685,30 @@ class TestBioRatL5Tpc4:
     def test_with_rng(self):
         _test_full(
             "path_distances",
-            "bio_rat_L5_TPC_B.json",
+            "bio_rat_L5_TPC_B_distribution.json",
             "params4.json",
             "expected_bio_rat_L5_TPC_B_with_params4.h5",
             "expected_bio_rat_L5_TPC_B_with_params4_persistence_diagram.json",
             rng_or_seed=build_random_generator(0),
         )
 
-    def test_skip_validation(self):
+    def test_skip_preprocessing(self):
         _test_full(
             "path_distances",
-            "bio_rat_L5_TPC_B.json",
+            "bio_rat_L5_TPC_B_distribution.json",
             "params4_orientation_manager.json",
             "expected_bio_rat_L5_TPC_B_with_params4.h5",
             "expected_bio_rat_L5_TPC_B_with_params4_persistence_diagram.json",
-            skip_validation=True,
+            skip_preprocessing=True,
         )
 
-    def test_skip_rng_and_validation(self):
+    def test_skip_rng_and_preprocessing(self):
         _test_full(
             "path_distances",
-            "bio_rat_L5_TPC_B.json",
+            "bio_rat_L5_TPC_B_distribution.json",
             "params4_orientation_manager.json",
             "expected_bio_rat_L5_TPC_B_with_params4.h5",
             "expected_bio_rat_L5_TPC_B_with_params4_persistence_diagram.json",
-            skip_validation=True,
+            skip_preprocessing=True,
             rng_or_seed=build_random_generator(0),
         )
