@@ -69,14 +69,6 @@ class TestValidateParams:
         dummy_params["diameter_params"] = {"method": "default"}
         tested.validate_neuron_params(dummy_params)
 
-    def test_default_diameter_unknown_key(self, dummy_params):
-        dummy_params["diameter_params"] = {
-            "method": "default",
-            "other key": "any value",
-        }
-        with pytest.raises(tested.ValidationError):
-            tested.validate_neuron_params(dummy_params)
-
     def test_M1_diameter_unknown_key(self, dummy_params):
         dummy_params["diameter_params"] = {"method": "M1", "other key": "any value"}
         with pytest.raises(tested.ValidationError):
@@ -105,6 +97,14 @@ class TestValidateParams:
         dummy_params["unknown_param"] = 0
         tested.validate_neuron_params(dummy_params)
 
+    def test_error_in_list(self, dummy_params):
+        # Wrong element in lists should be reported correctly
+        dummy_params["grow_types"] = ["UNKNOWN TYPE"]
+        with pytest.raises(
+            tested.ValidationError, match=r"In \[grow_types->0\]: 'UNKNOWN TYPE' is not one of"
+        ):
+            tested.validate_neuron_params(dummy_params)
+
 
 def test_empty_params():
     data = {
